@@ -17,7 +17,7 @@
 
 package Lemonldap::NG::Manager::Build::Tree;
 
-our $VERSION = '2.16.1';
+our $VERSION = '2.21.0';
 
 sub tree {
     return [ {
@@ -54,7 +54,9 @@ sub tree {
                                 'portalFavicon',
                                 'showLanguages',
                                 'scrollTop',
+                                'floatingCategoryName',
                                 'portalCustomCss',
+                                'portalCustomJs',
                                 'portalSkin',
                                 'portalSkinBackground',
                                 'portalSkinRules',
@@ -89,6 +91,7 @@ sub tree {
                                         'passwordPolicyActivation',
                                         'portalDisplayPasswordPolicy',
                                         'passwordPolicyMinSize',
+                                        'passwordPolicyMaxSize',
                                         'passwordPolicyMinLower',
                                         'passwordPolicyMinUpper',
                                         'passwordPolicyMinDigit',
@@ -109,6 +112,7 @@ sub tree {
                                         'portalErrorOnExpiredSession',
                                         'portalErrorOnMailNotFound',
                                         'portalDisplayRefreshMyRights',
+                                        'cacheTagSalt',
                                     ]
                                 }
                             ]
@@ -149,7 +153,8 @@ sub tree {
                             help  => 'authchoice.html',
                             nodes => [
                                 'authChoiceParam',     'authChoiceModules',
-                                'authChoiceAuthBasic', 'authChoiceFindUser'
+                                'authChoiceAuthBasic', 'authChoiceFindUser',
+                                'authChoiceSelectOnly',
                             ]
                         },
                         {
@@ -173,24 +178,10 @@ sub tree {
                                 {
                                     title => 'dbiConnection',
                                     help  => 'authdbi.html#connection',
-                                    nodes => [ {
-                                            title => 'dbiConnectionAuth',
-                                            form  => 'simpleInputContainer',
-                                            nodes => [
-                                                'dbiAuthChain',
-                                                'dbiAuthUser',
-                                                'dbiAuthPassword'
-                                            ]
-                                        },
-                                        {
-                                            title => 'dbiConnectionUser',
-                                            form  => 'simpleInputContainer',
-                                            nodes => [
-                                                'dbiUserChain',
-                                                'dbiUserUser',
-                                                'dbiUserPassword'
-                                            ]
-                                        }
+                                    form  => 'simpleInputContainer',
+                                    nodes => [
+                                        'dbiAuthChain', 'dbiAuthUser',
+                                        'dbiAuthPassword'
                                     ]
                                 },
                                 {
@@ -250,6 +241,12 @@ sub tree {
                                 'krbAllowedDomains',
                             ]
                         },
+                        {
+                            title => 'webauthnParams',
+                            form  => 'simpleInputContainer',
+                            help  => 'authwebauthn.html',
+                            nodes => [ 'webauthnAuthnLevel', ]
+                        },
                         {
                             title => 'ldapParams',
                             help  => 'authldap.html',
@@ -284,6 +281,7 @@ sub tree {
                                     form  => 'simpleInputContainer',
                                     nodes => [
                                         'ldapGroupBase',
+                                        'groupLDAPFilter',
                                         'ldapGroupObjectClass',
                                         'ldapGroupAttributeName',
                                         'ldapGroupAttributeNameUser',
@@ -397,9 +395,14 @@ sub tree {
                             title => 'radiusParams',
                             help  => 'authradius.html',
                             nodes => [
-                                'radiusAuthnLevel', 'radiusSecret',
-                                'radiusServer',     'radiusExportedVars',
+                                'radiusAuthnLevel',
+                                'radiusSecret',
+                                'radiusServer',
+                                'radiusTimeout',
+                                'radiusMsgAuth',
+                                'radiusExportedVars',
                                 'radiusDictionaryFile',
+                                'radiusRequestAttributes',
                             ]
                         },
                         {
@@ -475,6 +478,15 @@ sub tree {
                     title => 'issuerParams',
                     help  => 'start.html#identity-provider',
                     nodes => [ {
+                            title => 'issuerDBCAS',
+                            help  => 'idpcas.html#enabling-cas',
+                            form  => 'simpleInputContainer',
+                            nodes => [
+                                'issuerDBCASActivation', 'issuerDBCASPath',
+                                'issuerDBCASRule',
+                            ]
+                        },
+                        {
                             title => 'issuerDBSAML',
                             help  => 'idpsaml.html',
                             form  => 'simpleInputContainer',
@@ -484,12 +496,13 @@ sub tree {
                             ]
                         },
                         {
-                            title => 'issuerDBCAS',
-                            help  => 'idpcas.html#enabling-cas',
+                            title => 'issuerDBOpenIDConnect',
+                            help  => 'idpopenidconnect.html',
                             form  => 'simpleInputContainer',
                             nodes => [
-                                'issuerDBCASActivation', 'issuerDBCASPath',
-                                'issuerDBCASRule',
+                                'issuerDBOpenIDConnectActivation',
+                                'issuerDBOpenIDConnectPath',
+                                'issuerDBOpenIDConnectRule',
                             ]
                         },
                         {
@@ -524,16 +537,6 @@ sub tree {
                                 }
                             ]
                         },
-                        {
-                            title => 'issuerDBOpenIDConnect',
-                            help  => 'idpopenidconnect.html',
-                            form  => 'simpleInputContainer',
-                            nodes => [
-                                'issuerDBOpenIDConnectActivation',
-                                'issuerDBOpenIDConnectPath',
-                                'issuerDBOpenIDConnectRule',
-                            ]
-                        },
                         {
                             title => 'issuerDBGet',
                             help  => 'issuerdbget.html',
@@ -544,6 +547,34 @@ sub tree {
                                 'issuerDBGetParameters'
                             ]
                         },
+                        {
+                            title => 'issuerDBJitsiMeetTokens',
+                            help  => 'idpjitsimeettokens.html',
+                            nodes => [
+                                'issuerDBJitsiMeetTokensActivation',
+                                'issuerDBJitsiMeetTokensPath',
+                                'issuerDBJitsiMeetTokensRule',
+                                {
+                                    title => 'jitsiOptions',
+                                    nodes => [
+                                        'jitsiDefaultServer',
+                                        'jitsiAppId',
+                                        'jitsiAppSecret',
+                                        'jitsiSigningAlg',
+                                        'jitsiExpiration',
+                                        {
+                                            title => 'jitsiAttributes',
+                                            form  => 'simpleInputContainer',
+                                            nodes => [
+                                                'jitsiIdAttribute',
+                                                'jitsiNameAttribute',
+                                                'jitsiMailAttribute',
+                                            ]
+                                        }
+                                    ]
+                                }
+                            ]
+                        },
                         {
                             title => 'issuerOptions',
                             help  => 'start.html#options',
@@ -564,10 +595,10 @@ sub tree {
                     help  => 'ssocookie.html',
                     form  => 'simpleInputContainer',
                     nodes => [
-                        'cookieName', '*domain',
-                        'cda',        'securedCookie',
-                        'httpOnly',   'cookieExpiration',
-                        'sameSite',
+                        'cookieName',    '*domain',
+                        'cda',           'httpOnly',
+                        'securedCookie', 'sameSite',
+                        'cookieExpiration',
                     ]
                 },
                 {
@@ -575,11 +606,13 @@ sub tree {
                     help  => 'sessions.html',
                     nodes => [
                         'storePassword',
+                        'storePasswordEncrypted',
                         'displaySessionId',
                         'timeout',
                         'timeoutActivity',
                         'timeoutActivityInterval',
                         'grantSessionRules',
+                        'appAccessHistoryEnabled',
                         {
                             title => 'sessionStorage',
                             help  => 'start.html#sessions-database',
@@ -609,11 +642,6 @@ sub tree {
                         },
                     ]
                 },
-                {
-                    title => 'reloadParams',
-                    help  => 'configlocation.html#configuration-reload',
-                    nodes => [ 'reloadTimeout', 'compactConf', 'reloadUrls' ]
-                },
                 {
                     title => 'plugins',
                     help  => 'start.html#plugins',
@@ -680,7 +708,7 @@ sub tree {
                             ]
                         },
                         {
-                            title => 'passwordManagement',
+                            title => 'passwordResetByMailManagement',
                             help  => 'resetpassword.html',
                             nodes => [ {
                                     title => 'mailContent',
@@ -734,14 +762,16 @@ sub tree {
                         'autoSigninRules',
                         {
                             title => 'stayConnect',
-                            help  => 'stayconnected.html',
+                            help  => 'trustedbrowser.html',
                             form  => 'simpleInputContainer',
                             nodes => [
+                                'trustedBrowserRule',
                                 'stayConnected',
                                 'stayConnectedBypassFG',
                                 'stayConnectedTimeout',
                                 'stayConnectedCookieName',
                                 'stayConnectedSingleSession',
+                                'trustedBrowserUseTotp',
                             ],
                         },
                         {
@@ -754,6 +784,7 @@ sub tree {
                                 'rememberCookieTimeout',
                                 'rememberDefaultChecked',
                                 'rememberTimer',
+                                'rememberAuthChoiceForgetAtLogout',
                             ]
                         },
                         {
@@ -783,6 +814,7 @@ sub tree {
                             help  => 'notifications.html',
                             nodes => [
                                 'notification',
+                                'publicNotifications',
                                 'notificationsExplorer',
                                 'notificationWildcard',
                                 'oldNotifFormat',
@@ -902,29 +934,48 @@ sub tree {
                             nodes =>
                               [ 'decryptValueRule', 'decryptValueFunctions', ]
                         },
+                        {
+                            title => 'initializePasswordResetLabel',
+                            help  => 'initializePasswordReset.html',
+                            form  => 'simpleInputContainer',
+                            nodes => [
+                                'initializePasswordReset',
+                                'initializePasswordResetSecret',
+                            ],
+                        },
                         {
                             title => 'customPluginsNode',
-                            help  => 'plugincustom.html',
-                            nodes => [ 'customPlugins', 'customPluginsParams' ]
+                            help  => 'plugincustom.html#plugin-options',
+                            nodes => [ 'customPlugins', 'customPluginsParams', 'disabledPlugins' ]
                         },
                     ]
                 },
                 {
                     title => 'secondFactors',
                     help  => 'secondfactor.html',
-                    nodes => [
-                        'sfManagerRule',
-                        'sfRequired',
-                        'sfOnlyUpgrade',
-                        'sfLoginTimeout',
-                        'sfRegisterTimeout',
-                        {
-                            title => 'utotp2f',
-                            help  => 'utotp2f.html',
+                    nodes => [ {
+                            title => 'sfOptions',
+                            help  => 'secondfactor.html',
                             form  => 'simpleInputContainer',
                             nodes => [
-                                'utotp2fActivation', 'utotp2fAuthnLevel',
-                                'utotp2fLabel',      'utotp2fLogo'
+                                'sfManagerRule',
+                                'sfRequired',
+                                'sfOnlyUpgrade',
+                                'sfLoginTimeout',
+                                'sfRegisterTimeout',
+                                'sfRetries',
+                                {
+                                    title => 'sfRemovedNotification',
+                                    help  => 'secondfactor.html',
+                                    form  => 'simpleInputContainer',
+                                    nodes => [
+                                        'sfRemovedMsgRule',
+                                        'sfRemovedUseNotif',
+                                        'sfRemovedNotifRef',
+                                        'sfRemovedNotifTitle',
+                                        'sfRemovedNotifMsg'
+                                    ],
+                                }
                             ]
                         },
                         {
@@ -946,17 +997,6 @@ sub tree {
                                 'totp2fTTL'
                             ]
                         },
-                        {
-                            title => 'u2f',
-                            help  => 'u2f.html',
-                            form  => 'simpleInputContainer',
-                            nodes => [
-                                'u2fActivation',       'u2fSelfRegistration',
-                                'u2fUserCanRemoveKey', 'u2fAuthnLevel',
-                                'u2fLabel',            'u2fLogo',
-                                'u2fTTL'
-                            ]
-                        },
                         {
                             title => 'webauthn2f',
                             help  => 'webauthn2f.html',
@@ -964,10 +1004,16 @@ sub tree {
                             nodes => [
                                 'webauthn2fActivation',
                                 'webauthn2fSelfRegistration',
-                                'webauthn2fUserVerification',
                                 'webauthn2fUserCanRemoveKey',
+                                'webauthn2fUserVerification',
+                                'webauthn2fResidentKey',
+                                'webauthn2fAttestation',
+                                'webauthn2fAttestationTrust',
                                 'webauthnRpName',
+                                'webauthnRpId',
+                                'webauthnAppId',
                                 'webauthnDisplayNameAttr',
+                                'webauthnDefaultTransports',
                                 'webauthn2fAuthnLevel',
                                 'webauthn2fLabel',
                                 'webauthn2fLogo',
@@ -1030,6 +1076,18 @@ sub tree {
                                 'ext2fLabel',          'ext2fLogo',
                             ]
                         },
+                        {
+                            title => 'okta2f',
+                            help  => 'okta2f.html',
+                            form  => 'simpleInputContainer',
+                            nodes => [
+                                'okta2fActivation', 'okta2fAdminURL',
+                                'okta2fApiKey',     'okta2fLoginAttribute',
+                                'okta2fAuthnLevel', 'okta2fLabel',
+                                'okta2fLogo',
+                            ]
+                        },
+
                         {
                             title => 'radius2f',
                             help  => 'radius2f.html',
@@ -1040,6 +1098,10 @@ sub tree {
                                 'radius2fSecret',
                                 'radius2fUsernameSessionKey',
                                 'radius2fTimeout',
+                                'radius2fMsgAuth',
+                                'radius2fDictionaryFile',
+                                'radius2fRequestAttributes',
+                                'radius2fSendInitialRequest',
                                 'radius2fAuthnLevel',
                                 'radius2fLabel',
                                 'radius2fLogo'
@@ -1063,16 +1125,6 @@ sub tree {
                             ]
                         },
                         'sfExtra',
-                        {
-                            title => 'sfRemovedNotification',
-                            help  => 'secondfactor.html',
-                            form  => 'simpleInputContainer',
-                            nodes => [
-                                'sfRemovedMsgRule',  'sfRemovedUseNotif',
-                                'sfRemovedNotifRef', 'sfRemovedNotifTitle',
-                                'sfRemovedNotifMsg'
-                            ],
-                        }
                     ]
                 },
                 {
@@ -1121,13 +1173,17 @@ sub tree {
                                 'formTimeout',
                                 'tokenUseGlobalStorage',
                                 'strictTransportSecurityMax_Age',
+                                'hashedSessionStore',
                                 {
                                     title => 'CrowdSecPlugin',
                                     help  => 'crowdsec.html',
                                     form  => 'simpleInputContainer',
                                     nodes => [
-                                        'crowdsec',    'crowdsecAction',
-                                        'crowdsecUrl', 'crowdsecKey',
+                                        'crowdsec',
+                                        'crowdsecAction',
+                                        'crowdsecUrl',
+                                        'crowdsecKey',
+                                        'crowdsecIgnoreFailures',
                                     ],
                                 },
                                 {
@@ -1139,6 +1195,16 @@ sub tree {
                                         'checkHIBPRequired'
                                     ],
                                 },
+                                {
+                                    title => 'Entropycheck',
+                                    help  => 'checkentropy.html',
+                                    form  => 'simpleInputContainer',
+                                    nodes => [
+                                        'checkEntropy',
+                                        'checkEntropyRequired',
+                                        'checkEntropyRequiredLevel'
+                                    ],
+                                },
                                 {
                                     title => 'newLocationWarnings',
                                     help  => 'newlocationwarning.html',
@@ -1203,6 +1269,7 @@ sub tree {
                             nodes => [
                                 'https',
                                 'port',
+                                'useRedirectAjaxOnUnauthorized',
                                 'useRedirectOnForbidden',
                                 'useRedirectOnError',
                                 'maintenance'
@@ -1229,6 +1296,15 @@ sub tree {
                                 'redirectFormMethod', 'activeTimer',
                             ]
                         },
+                        {
+                            title => 'eventsManagement',
+                            help  => 'eventsmanagement.html',
+                            nodes => [
+                                'messageBroker', 'messageBrokerOptions',
+                                'reloadTimeout', 'reloadUrls',
+                                'compactConf',   'eventQueueName',
+                            ]
+                        },
                     ]
                 }
             ]
@@ -1241,6 +1317,7 @@ sub tree {
         'virtualHosts',
         {
             title => 'samlServiceMetaData',
+            form  => 'displaySamlMetadata',
             help  => 'samlservice.html',
             nodes => [
                 'samlEntityID',
@@ -1288,7 +1365,8 @@ sub tree {
                         'samlAuthnContextMapPassword',
                         'samlAuthnContextMapPasswordProtectedTransport',
                         'samlAuthnContextMapKerberos',
-                        'samlAuthnContextMapTLSClient'
+                        'samlAuthnContextMapTLSClient',
+                        'samlAuthnContextMapExtra'
                     ]
                 },
                 {
@@ -1416,11 +1494,13 @@ sub tree {
         'samlSPMetaDataNodes',
         {
             title => 'oidcServiceMetaData',
+            form  => 'displayOidcMetadata',
             help  => 'openidconnectservice.html#service-configuration',
             nodes => [
                 'oidcServiceMetaDataIssuer',
                 {
                     title => 'oidcServiceMetaDataEndPoints',
+                    help  => 'openidconnectservice.html#endpoints',
                     form  => 'simpleInputContainer',
                     nodes => [
                         'oidcServiceMetaDataAuthorizeURI',
@@ -1436,8 +1516,10 @@ sub tree {
                     ]
                 },
                 'oidcServiceMetaDataAuthnContext',
+                'oidcServiceMetaDataAmrRules',
                 {
                     title => "oidcServiceDynamicRegistration",
+                    help  => 'openidconnectservice.html#dynamic-registration',
                     nodes => [
                         'oidcServiceAllowDynamicRegistration',
                         'oidcServiceDynamicRegistrationExportedVars',
@@ -1446,34 +1528,70 @@ sub tree {
                 },
                 {
                     title => 'oidcServiceMetaDataSecurity',
+                    help  => 'openidconnectservice.html#security',
                     nodes => [ {
                             title => 'oidcServiceMetaDataKeys',
-                            form  => 'RSACertKeyNoPassword',
-                            group => [
-                                'oidcServicePrivateKeySig',
-                                'oidcServicePublicKeySig',
-                                'oidcServiceKeyIdSig',
+                            nodes => [ {
+                                    title => 'oidcServiceMetaDataSigKeys',
+                                    form  => 'OidcKey',
+                                    group => [
+                                        'oidcServicePrivateKeySig',
+                                        'oidcServicePublicKeySig',
+                                        'oidcServiceKeyIdSig',
+                                        'oidcServiceKeyTypeSig',
+                                        'oidcServiceOldPrivateKeySig',
+                                        'oidcServiceOldPublicKeySig',
+                                        'oidcServiceOldKeyIdSig',
+                                        'oidcServiceOldKeyTypeSig',
+                                        'oidcServiceNewPrivateKeySig',
+                                        'oidcServiceNewPublicKeySig',
+                                        'oidcServiceNewKeyIdSig',
+                                        'oidcServiceNewKeyTypeSig',
+                                    ],
+                                },
+                                {
+                                    title => 'oidcServiceMetaDataEncKeys',
+                                    form  => 'OidcKey',
+                                    group => [
+                                        'oidcServicePrivateKeyEnc',
+                                        'oidcServicePublicKeyEnc',
+                                        'oidcServiceKeyIdEnc',
+                                        'oidcServiceKeyTypeEnc',
+                                        'oidcServiceOldPrivateKeyEnc',
+                                        'oidcServiceOldPublicKeyEnc',
+                                        'oidcServiceOldKeyIdEnc',
+                                        'oidcServiceOldKeyTypeEnc',
+                                    ],
+                                },
                             ],
                         },
                         'oidcServiceAllowAuthorizationCodeFlow',
                         'oidcServiceAllowImplicitFlow',
                         'oidcServiceAllowHybridFlow',
-                        'oidcServiceAllowOnlyDeclaredScopes',
                         'oidcServiceIgnoreScopeForClaims',
+                        'oidcServiceAllowOnlyDeclaredScopes',
+                        'oidcDropCspHeaders',
+                        'oidcServiceEncAlgorithmAlg',
+                        'oidcServiceEncAlgorithmEnc',
+                        'oidcServiceHideMetadata',
+                        'oidcServiceMetaDataDisallowNoneAlg',
                     ],
                 },
                 {
                     title => 'oidcServiceMetaDataTimeouts',
+                    help  => 'openidconnectservice.html#timeouts',
                     form  => 'simpleInputContainer',
                     nodes => [
                         'oidcServiceAuthorizationCodeExpiration',
                         'oidcServiceIDTokenExpiration',
                         'oidcServiceAccessTokenExpiration',
                         'oidcServiceOfflineSessionExpiration',
+                        'oidcServiceMetadataTtl',
                     ]
                 },
                 {
                     title => "oidcServiceMetaDataSessions",
+                    help  => 'openidconnectservice.html#sessions',
                     nodes => [ 'oidcStorage', 'oidcStorageOptions' ],
                 },
             ]
@@ -1484,10 +1602,14 @@ sub tree {
             title => 'casServiceMetadata',
             help  => 'idpcas.html#configuring-the-cas-service',
             nodes => [
-                'casAttr',                    'casAccessControlPolicy',
-                'casStrictMatching',          'casTicketExpiration',
-                'casBackChannelSingleLogout', 'casStorage',
-                'casStorageOptions',          'casAttributes',
+                'casAttr',
+                'casAccessControlPolicy',
+                'casStrictMatching',
+                'casTicketExpiration',
+                'casBackChannelSingleLogout',
+                'casStorage',
+                'casStorageOptions',
+                'casAttributes',
             ]
         },
         'casSrvMetaDataNodes',
